#! /bin/sh
#  ------------------------------------------------------------
#  Company: Software- und Organisations-Service GmbH
#  Purpose: Startscript for JS7 Controller
#  ------------------------------------------------------------

### BEGIN INIT INFO
# Provides:          JS7Controller
# Required-Start:    $syslog $remote_fs
# Required-Stop:     $syslog $remote_fs
# Default-Start:     3 5
# Default-Stop:      0 1 2 6
# Description:       Start JS7 Controller
### END INIT INFO

### USAGE OF THIS FILE ##################################
#
# It can be used as service startscript in /etc/init.d
#
# Each instance of the JS7 Controller must have a
# different Id. For example if the Id 'myscheduler' 
# is used for the instance then copy this file
# 
# 'controller.sh' 
# -> 'controller_myscheduler.sh'
#
# and set the JS7_CONTROLLER_ID variable below.
#
# If you move this script outside the bin directory of 
# the JS7 Controller installation path 
# (e.g. /etc/init.d) then you have to set JS7_CONTROLLER_HOME.
#  
# Further set the JS7_CONTROLLER_USER variable.
#
##########################################################


### SETTINGS #############################################

# This variable has to point to the installation path of 
# the JS7 Controller. If this variable not defined then the parent
# directory of this startscript is used.
### NOTE: 
# This variable is required if this script is used for a  
# JS7 Controller service in /etc/init.d to find the installation 
# of the JS7 Controller.
#
# JS7_CONTROLLER_HOME=


# Sets the user for the JS7 Controller.
# Without this setting the currently logged in user is used.
# The JS7 Controller should always be started under the same user
# to avoid problems about permissions in the JS7_CONTROLLER_DATA folder.
# This variable has to be set if this script is used for autostart
# to avoid that the JS7 Controller is started as the 'root' user.
#
# JS7_CONTROLLER_USER=


# The default JS7 Controller Id

test -z "${JS7_CONTROLLER_ID}" && JS7_CONTROLLER_ID=controller


# The default port of the JS7 Controller

JS7_CONTROLLER_DEFAULT_HTTP_PORT=4444

# Sets the http port for the JS7 Controller.
# Without this setting the default port 4444 is used.
# If just a port is specified then the JS7 Controller listens to all
# available network interfaces. This corresponds to 0.0.0.0:<port>.
# Use the form <ip address or hostname>:<port> to indicate
# a specific network interface the JS7 Controller should listen to.
# The command line option --http-port beats the environment
# variable JS7_CONTROLLER_HTTP_PORT.
#
# JS7_CONTROLLER_HTTP_PORT=


# In addition to the http port an https port for the
# JS7 Controller can be specified. If just a port is specified
# then the JS7 Controller listens to all available network interfaces.
# This corresponds to using 0.0.0.0:<port>.
# Use the form <ip address or hostname>:<port> to indicate
# a specific network interface the JS7 Controller should listen to.
# The command line option --https-port beats the environment
# variable JS7_CONTROLLER_HTTPS_PORT.
#
# JS7_CONTROLLER_HTTPS_PORT=


# Sets the directory used by the JS7 Controller for configuration
# files, log files, journal files and temporary files.
# This directory has to be unique for any instances of the
# JS7 Controller. The default value is JS7_CONTROLLER_HOME\var
# Check that the JS7 Controller user account has read/write
# permissions for this directory.
#
# JS7_CONTROLLER_DATA=


# Sets the directory used by the JS7 Controller for configuration
# files. The default value is JS7_CONTROLLER_DATA/config
# The command line option --config-directory beats the environment
# variable JS7_CONTROLLER_CONFIG_DIR
# Check that the JS7 Controller user account has read/write
# permissions for this directory.
#
# JS7_CONTROLLER_CONFIG_DIR=


# Sets the directory to which the JS7 Controller stores log files.
# The default value is JS7_CONTROLLER_DATA/logs
# Check that the JS7 Controller user account has read/write
# permissions for this directory.
#
# JS7_CONTROLLER_LOGS=


# Sets the directory to which the JS7 Controller stores its PID file.
# The default value is JS7_CONTROLLER_LOGS
# Check that the JS7 Controller user account has read/write
# permissions for this directory.
#
# JS7_CONTROLLER_PID_FILE_DIR=


# Specifies the Java Runtime Environment to be used by the
# JS7 Controller. If JAVA_HOME is not set then Java is used from
# the PATH.
#
# JAVA_HOME=


# With Java 1.8 initial memory allocation has changed,
# for details see https://kb.sos-berlin.com/x/aIC9
# As a result on start-up of the JS7 Controller the amount of
# virtual memory assigned by Java depends on the physical
# memory and in most cases is by far too high.
# The environment variables JAVA_OPTIONS can be used to apply
# memory settings.
# The default value for JAVA_OPTIONS is -Xmx500m
#
# JAVA_OPTIONS=

##########################################################


usage() 
{
  echo "Usage: `basename $0` command [options]"
  echo "  command:"
  echo "    start         [options]"
  echo "    start_docker  [options]"
  echo "    stop          [options]"
  echo "    abort         [options]"
  echo "    restart       [options]"
  echo "    status        [options]"
  echo "    kill          [options]"
  echo "    cert          [cert-options]                    | see https://kb.sos-berlin.com/x/jLbAAw"
  echo "  options:"
  echo "    --id=<JS7 Controller Id>                        | default: $JS7_CONTROLLER_ID"
  echo "    --http-port=<[hostname or ip address:]number>   | default: $JS7_CONTROLLER_HTTP_PORT"
  echo "    --https-port=<[hostname or ip address:]number>  | default: $JS7_CONTROLLER_HTTPS_PORT"
  echo "    --data-directory=<location of data directory>   | default: $JS7_CONTROLLER_DATA"
  echo "    --config-directory=<location of data directory> | default: $JS7_CONTROLLER_CONFIG_DIR"
  echo "    --java-options=<java options>                   | default: ${JAVA_OPTIONS}; see https://kb.sos-berlin.com/x/aIC9"
  echo "    -c | --curl                                     | use curl instead of java http client"
  more_info
}

more_info()
{
  echo "see https://kb.sos-berlin.com/x/fwmGAw for more information."
}

start_controller() 
{
  JS7_CONTROLLER_EXIT=0
  if running
  then
    echo "...JS7 Controller(${JS7_CONTROLLER_ID}) is already running with pid=`get_pid`!"
    exit 0
  fi
  JS7_CONTROLLER_CLASSPATH="${JS7_CONTROLLER_CONFIG_DIR}/lib/*:${JS7_CONTROLLER_HOME}/lib/patches/*:${JS7_CONTROLLER_HOME}/lib/user_lib/*:${JS7_CONTROLLER_HOME}/lib/sos/*:${JS7_CONTROLLER_HOME}/lib/3rd-party/*"
  if [ -f "${JS7_CONTROLLER_CONFIG_DIR}/log4j2.xml" ]
  then
    JS7_CONTROLLER_CLASSPATH="${JS7_CONTROLLER_CLASSPATH}:${JS7_CONTROLLER_CONFIG_DIR}"
  else
    JS7_CONTROLLER_CLASSPATH="${JS7_CONTROLLER_CLASSPATH}:${JS7_CONTROLLER_HOME}/lib"
  fi
  if [ -d "${JS7_CONTROLLER_CONFIG_DIR}/user_lib" ]
  then
    JS7_CONTROLLER_CLASSPATH="${JS7_CONTROLLER_CONFIG_DIR}/user_lib/*:${JS7_CONTROLLER_CLASSPATH}"
  fi
  if [ ! -z "$JS7_CONTROLLER_TZ" ]
  then
    JS7_CONTROLLER_TZ=`get_timezone`
  fi
  CUR_DIR="`pwd`"
  cd "${JS7_CONTROLLER_DATA}"
  export_env
  
  "${JS7_CONTROLLER_HOME}/bin/controller_watchdog.sh" -${JS7_CONTROLLER_ID} &
  JS7_CONTROLLER_EXIT=$?
  cd "${CUR_DIR}"
}

start_controller_in_docker()
{
  JS7_CONTROLLER_EXIT=0
  if running
  then
     echo "...JS7 Controller(${JS7_CONTROLLER_ID}) is already running with pid=`get_pid`!"
     exit 0
  fi
  JS7_CONTROLLER_CLASSPATH="${JS7_CONTROLLER_CONFIG_DIR}/lib/*:${JS7_CONTROLLER_HOME}/lib/patches/*:${JS7_CONTROLLER_HOME}/lib/user_lib/*:${JS7_CONTROLLER_HOME}/lib/sos/*:${JS7_CONTROLLER_HOME}/lib/3rd-party/*"
  if [ -f "${JS7_CONTROLLER_CONFIG_DIR}/log4j2.xml" ]
  then
    JS7_CONTROLLER_CLASSPATH="${JS7_CONTROLLER_CLASSPATH}:${JS7_CONTROLLER_CONFIG_DIR}"
  else
    JS7_CONTROLLER_CLASSPATH="${JS7_CONTROLLER_CLASSPATH}:${JS7_CONTROLLER_HOME}/lib"
  fi
  if [ -d "${JS7_CONTROLLER_CONFIG_DIR}/user_lib" ]
  then
    JS7_CONTROLLER_CLASSPATH="${JS7_CONTROLLER_CONFIG_DIR}/user_lib/*:${JS7_CONTROLLER_CLASSPATH}"
  fi
  if [ ! -z "$JS7_CONTROLLER_TZ" ]
  then
    JS7_CONTROLLER_TZ=`get_timezone`
  fi
  CUR_DIR="`pwd`"
  cd "${JS7_CONTROLLER_DATA}"
  export_env
  if [ "`uname`" != "AIX" ]
  then 
  	trap "stop_controller" SIGINT SIGTERM
  fi
  
  remove_folder "${JS7_CONTROLLER_DATA}/work"
  create_folder "${JS7_CONTROLLER_DATA}/work"
  
  "${JAVABIN}" ${JAVA_OPTIONS} -classpath "${JS7_CONTROLLER_CLASSPATH}" js7.controller.ControllerMain --id=${JS7_CONTROLLER_ID} --http-port=${JS7_CONTROLLER_HTTP_PORT} ${HTTPS_PORT_OPTION} --config-directory="${JS7_CONTROLLER_CONFIG_DIR}" --data-directory="${JS7_CONTROLLER_DATA}" > "${JS7_CONTROLLER_LOGS}/watchdog.log" 2>&1 &
  JS7_CONTROLLER_EXIT=$?
  cd "${CUR_DIR}"
  if [ "${JS7_CONTROLLER_EXIT}" = "0" ]
  then
    create_pid_file $!
    JS7_CONTROLLER_PID=$!
    echo "...JS7 Controller(${JS7_CONTROLLER_ID}) is started with pid=${JS7_CONTROLLER_PID}!"
    wait "${JS7_CONTROLLER_PID}"
    JS7_CONTROLLER_EXIT=$?
    restart_docker
  fi
}

restart_docker()
{
  rm "${JS7_CONTROLLER_PID_FILE}" >/dev/null 2>/dev/null
  
  #abort and restart
  if [ ${JS7_CONTROLLER_EXIT} -eq 98 ]
  then
    start_controller_in_docker
  fi
  #terminate and restart
  if [ ${JS7_CONTROLLER_EXIT} -eq 97 ]
  then
    start_controller_in_docker
  fi
}

stop_controller() 
{
  if running
  then
    echo "...try to terminate the JS7 Controller(${JS7_CONTROLLER_ID}) with pid=${JS7_CONTROLLER_PID}."
    http_command POST "{\"TYPE\": \"ShutDown\"}"
  else
    echo "...not started!"
    return 0 
  fi
  if [ "${JS7_CONTROLLER_EXIT}" != "0" ]
  then
    echo "...could not terminate the JS7 Controller(${JS7_CONTROLLER_ID}) with pid=${JS7_CONTROLLER_PID}!"
    exit 3
  else
    LOOP_COUNTER=2
    sleep 2
    while running
    do
      test "${LOOP_COUNTER}" = "2" && printf "...waiting for termination"
      printf ".."
      #printf "..%d" "${LOOP_COUNTER}"
      LOOP_COUNTER=`expr ${LOOP_COUNTER} + 2`
      sleep 2
    done
    echo "...terminated!"
    remove_pid_file
    return 0
  fi
  return 0
}

abort_controller() 
{
  if running
  then
    echo "...try to abort the JS7 Controller(${JS7_CONTROLLER_ID}) with pid=${JS7_CONTROLLER_PID}."
    http_command POST "{\"TYPE\": \"EmergencyStop\"}"
  else
    echo "...not started!"
    return 0 
  fi
  sleep 2
  if running
  then
    echo "...could not abort the JS7 Controller(${JS7_CONTROLLER_ID}) with pid=${JS7_CONTROLLER_PID}!"
    exit ${JS7_CONTROLLER_EXIT}
  else
    JS7_CONTROLLER_EXIT=0
    echo "...aborted!"
    remove_pid_file
    return 0
  fi
  return 0
}

status_controller() 
{  
  http_command GET "?"
  #exit 3 if program is not running according LSB 3.1
  if [ "${JS7_CONTROLLER_EXIT}" != "0" ]
  then
    echo "...JS7 Controller(${JS7_CONTROLLER_ID}) not started?"
    JS7_CONTROLLER_EXIT=3
  fi
}

kill_controller() 
{
  if running
  then
    JS7_CONTROLLER_PID=`get_pid`
    echo "...try to kill (SIGTERM) the JS7 Controller(${JS7_CONTROLLER_ID}) with pid=${JS7_CONTROLLER_PID}."
    kill -15 ${JS7_CONTROLLER_PID} 2> /dev/null
    sleep 5
    if ps -p ${JS7_CONTROLLER_PID} > /dev/null 2>&1
    then
      echo "...try once more with SIGKILL."
      kill -9 ${JS7_CONTROLLER_PID}
      kill_err=$?
      if [ $kill_err -eq 0 ]
      then
        echo "...killed!"
        remove_pid_file
        return 0
      else
        echo "...could not kill the JS7 Controller(${JS7_CONTROLLER_ID}) with pid=${JS7_CONTROLLER_PID}!"
        exit 3
      fi
    else
      echo "...killed!"
      remove_pid_file
      return 0 
    fi
  else
    echo "...not started!" 
  fi
  return 0 
}

call_cert() 
{
	JS7_CONTROLLER_LIBDIR="${JS7_CONTROLLER_HOME}/lib"
    JS7_CONTROLLER_LIBEXTDIR="${JS7_CONTROLLER_LIBDIR}/ext"
    "${JAVABIN}" -Xmx50m -classpath "${JS7_CONTROLLER_LIBDIR}/patches/*:${JS7_CONTROLLER_LIBEXTDIR}/sos/*:${JS7_CONTROLLER_LIBEXTDIR}/3rd-party/*:${JS7_CONTROLLER_LIBDIR}/3rd-party/*:${JS7_CONTROLLER_LIBDIR}/http-client/" -Djs7.config-directory="%JS7_CONTROLLER_CONFIG_DIR%" com.sos.cli.ExecuteRollOut "@*"
	exit $?
}

http_command()
{
  if with_curl
  then
    command_with_curl "$1" "$2"
  else
    command_with_controller_client "$2"
  fi
}

command_with_controller_client()
{
  JS7_CONTROLLER_EXIT=0
  JS7_CONTROLLER_CLASSPATH="${JS7_CONTROLLER_HOME}/lib/sos/*:${JS7_CONTROLLER_HOME}/lib/3rd-party/*:${JS7_CONTROLLER_HOME}/lib/http-client"
  "${JAVABIN}" -Xmx50m -classpath "${JS7_CONTROLLER_CLASSPATH}" js7.controller.client.main.ControllerClientMain --config-directory="${JS7_CONTROLLER_CONFIG_DIR}" --data-directory="${JS7_CONTROLLER_DATA}" "`get_url`" "$1" 2>/dev/null
  JS7_CONTROLLER_EXIT=$?
  return 0
}

command_with_curl()
{
  HTTP_METHOD="$1"
  if [ "$1" = "POST" ]
  then
    curl -L -X POST -d "$2" -H "X-JS7-Session: `cat ${JS7_CONTROLLER_DATA}/work/session-token`" -H "Content-Type: application/json" "`get_url`/controller/api/command"
  else
    curl -L -H "X-JS7-Session: `cat ${JS7_CONTROLLER_DATA}/work/session-token`" "`get_url`/controller/api$2"
  fi
  JS7_CONTROLLER_EXIT=$?
  #TODO if JS7_CONTROLLER_EXIT...
  return 0
}

get_pid() 
{
  if [ -r "${JS7_CONTROLLER_PID_FILE}" ]
  then
    cat "${JS7_CONTROLLER_PID_FILE}"
    return 0
  else
    JS7_CONTROLLER_PID=`get_pid_from_ps`
    echo "${JS7_CONTROLLER_PID}"
    if [ "${JS7_CONTROLLER_PID}" = "" ]
    then
      return 1
    fi
  fi 
  return 0 
}

get_pid_from_ps() 
{
  ps -ef -w -w | egrep "js7\.controller\.ControllerMain.*--http-port=${JS7_CONTROLLER_HTTP_PORT}" | grep -v "egrep" | awk '{print $2}'
}

running()
{
  JS7_CONTROLLER_PID=`get_pid 2>/dev/null` || return 1 
  if kill -0 ${JS7_CONTROLLER_PID} 2>/dev/null
  then
    return 0
  else
    remove_pid_file
  fi
  return 1
}

get_timezone()
{
  if [ -f "${JS7_CONTROLLER_HOME}/lib/classes/DefaultTimeZone.class" ]
  then
    "${JAVABIN}" -classpath "${JS7_CONTROLLER_HOME}/lib/classes" DefaultTimeZone 2>/dev/null
    test $? -eq 0 || echo "Etc/UTC"
  else
    echo "Etc/UTC"
  fi
}

with_curl()
{
  which curl >/dev/null 2>/dev/null || return 1
  return $WITH_CURL
}

create_pid_file() 
{
  JS7_CONTROLLER_PID=$1
  echo "${JS7_CONTROLLER_PID}" > "${JS7_CONTROLLER_PID_FILE}"
  if [ ! -f "${JS7_CONTROLLER_PID_FILE}" ]
  then
    echo "...pid file \"${JS7_CONTROLLER_PID_FILE}\" could not be created!"
    echo "...please check the permissions in \"${JS7_CONTROLLER_PID_FILE_DIR}\"."
    more_info
    #kill -9 ${JS7_CONTROLLER_PID}
    exit 3
  else
    chmod 666 "${JS7_CONTROLLER_PID_FILE}" 2>/dev/null
  fi 
}

remove_pid_file()
{
  rm -f "${JS7_CONTROLLER_PID_FILE}" 2>/dev/null
}

mkdir_error()
{
  if [ "$1" != "0" ]
  then 
    echo "...could not create \"$2\""
    echo "please create \"$2\" manually with read and write permissions for the user \"${JS7_CONTROLLER_USER}\""
    MKDIR_ERROR_EXISTS=1
    more_info
  fi
}

export_env()
{
  JS7_CONTROLLER_APPNAME=controller
  export JS7_CONTROLLER_ID
  export JS7_CONTROLLER_USER
  export JS7_CONTROLLER_APPNAME
  export JS7_CONTROLLER_HOME
  export JS7_CONTROLLER_DATA
  export JS7_CONTROLLER_CONFIG_DIR
  export JS7_CONTROLLER_PORT
  export JS7_CONTROLLER_HTTP_PORT
  export JS7_CONTROLLER_HTTPS_PORT
  # export HTTPS_PORT_OPTION
  export JAVABIN
  export JS7_CONTROLLER_LOGS
  export JAVA_OPTIONS
  export JS7_CONTROLLER_CLASSPATH
  export JS7_CONTROLLER_PID_FILE
  export JS7_CONTROLLER_PID_FILE_DIR
  export JS7_CONTROLLER_INSTANCESCRIPT
  export JS7_CONTROLLER_TZ
}

get_port()
{
  echo "$1" | cut -d : -f 2 
}

get_url()
{
  echo "http://`add_ip_address ${JS7_CONTROLLER_HTTP_PORT} 127.0.0.1`"
}

add_ip_address()
{
  IP_ADDRESS=`echo "$1" | grep ":" | cut -d : -f 1`
  NEW_IP_ADDRESS="$2"
  if [ ! -z "$NEW_IP_ADDRESS" ]
  then
    NEW_IP_ADDRESS="${NEW_IP_ADDRESS}:"
  fi
  if [ -z "${IP_ADDRESS}" ] || [ "${IP_ADDRESS}" = "0.0.0.0" ]
  then
    echo "${NEW_IP_ADDRESS}$1"
  else
    echo "$1"
  fi    
}

create_folder()
{
  if [ ! -d "$1" ]
  then
    if [ "${CURRENT_USER}" = "${JS7_CONTROLLER_USER}" ]
    then
      mkdir -p "$1" 2>/dev/null
    else
      su - ${JS7_CONTROLLER_USER} -c "mkdir -p \"$1\" 2>/dev/null"
    fi
    mkdir_error $? "$1"
  fi
}

remove_folder()
{
  if [ -d "$1" ]
  then
    if [ "${CURRENT_USER}" = "${JS7_CONTROLLER_USER}" ]
    then
      rm -Rf "$1" 2>/dev/null
    else
      su - ${JS7_CONTROLLER_USER} -c "rm -Rf \"$1\" 2>/dev/null"
    fi
  fi
}

copy_file()
{
  if [ -f "$1" ] && [ ! -f "$2" ]
  then
    if [ "${CURRENT_USER}" = "${JS7_CONTROLLER_USER}" ]
    then
      cp -f "$1" "$2" 2>/dev/null
    else
      su - ${JS7_CONTROLLER_USER} -c "cp -f \"$1\" \"$2\" 2>/dev/null"
    fi
    mkdir_error $? "$2"
  fi
}


##########################
# set and check home
JS7_CONTROLLER_BINDIR="`dirname $0`"
JS7_CONTROLLER_BINDIR="`cd \"${JS7_CONTROLLER_BINDIR}\" >/dev/null && pwd`"
JS7_CONTROLLER_STARTSCRIPT="${JS7_CONTROLLER_BINDIR}/`basename $0`"
if [ -z "${JS7_CONTROLLER_HOME}" ]
then
  JS7_CONTROLLER_HOME="`cd \"${JS7_CONTROLLER_BINDIR}/..\" >/dev/null && pwd`"
fi

if [ ! -d "${JS7_CONTROLLER_HOME}/lib" ]
then
  echo "...the directory \"${JS7_CONTROLLER_HOME}/lib\" is missing!"
  echo "did you move the startscript?"
  echo "please set the JS7_CONTROLLER_HOME variable in '$0'."
  more_info
  exit 3
fi


##########################
# set user
CURRENT_USER=`whoami 2>/dev/null` 
whoami_err=$?
# 'whoami' on Solaris sometimes not available
if [ ${whoami_err} -ne 0 ]
then
  CURRENT_USER=`id | sed 's/.*uid=[^(]*(\([^)]*\)).*/\1/'`
fi
if [ -z "${JS7_CONTROLLER_USER}" ]
then
  JS7_CONTROLLER_USER=${CURRENT_USER}
  #if [ "${CURRENT_USER}" = "root" ]
  #then
  #  echo "The current JS7_CONTROLLER_USER is 'root'!"
  #  echo "Please set the JS7_CONTROLLER_USER variable in '$0'."
  #  echo "If you really want the 'root' user then set"
  #  echo "JS7_CONTROLLER_USER=root in '$0'."
  #  more_info
  #  exit 2
  #fi
fi


##########################
# set http port etc.
HTTPS_PORT_OPTION=
JS7_CONTROLLER_PORT=
WITH_CURL=1
for param in "$@"
do
  case "$param" in
     --id=*)               JS7_CONTROLLER_ID=`echo "$param" | sed 's/--id=//'`
                           ;;
     --http-port=*)        JS7_CONTROLLER_HTTP_PORT=`echo "$param" | sed 's/--http-port=//'`
                           ;;
     --https-port=*)       JS7_CONTROLLER_HTTPS_PORT=`echo "$param" | sed 's/--https-port=//'`
                           ;;
     --data-directory=*)   JS7_CONTROLLER_DATA=`echo "$param" | sed 's/--data-directory=//'`
                           ;;
     --config-directory=*) JS7_CONTROLLER_CONFIG_DIR=`echo "$param" | sed 's/--config-directory=//'`
                           ;;
     --java-options=*)     JAVA_OPTIONS=`echo "$param" | sed 's/--java-options=//'`
                           ;;
     -c | --curl)          WITH_CURL=0
                           ;;
  esac
done

JS7_CONTROLLER_HTTP_PORT=`echo "${JS7_CONTROLLER_HTTP_PORT}" | sed 's/^://'`
JS7_CONTROLLER_HTTPS_PORT=`echo "${JS7_CONTROLLER_HTTPS_PORT}" | sed 's/^://'`
test -z "${JS7_CONTROLLER_HTTP_PORT}" && JS7_CONTROLLER_HTTP_PORT=`add_ip_address "${JS7_CONTROLLER_DEFAULT_HTTP_PORT}" "${JS7_CONTROLLER_IP_ADDRESS}"`
JS7_CONTROLLER_PORT=`get_port "${JS7_CONTROLLER_HTTP_PORT}"`
test -z "${JS7_CONTROLLER_HTTPS_PORT}" || HTTPS_PORT_OPTION="--https-port=${JS7_CONTROLLER_HTTPS_PORT}"



##########################
# set pid file
test -z "${JS7_CONTROLLER_DATA}" && JS7_CONTROLLER_DATA="${JS7_CONTROLLER_HOME}/var"
test -z "${JS7_CONTROLLER_CONFIG_DIR}" && JS7_CONTROLLER_CONFIG_DIR="${JS7_CONTROLLER_DATA}/config"
test -z "${JS7_CONTROLLER_LOGS}" && JS7_CONTROLLER_LOGS="${JS7_CONTROLLER_DATA}/logs"
test -z "${JS7_CONTROLLER_PID_FILE_DIR}" && JS7_CONTROLLER_PID_FILE_DIR="${JS7_CONTROLLER_LOGS}"
JS7_CONTROLLER_PID_FILE="${JS7_CONTROLLER_PID_FILE_DIR}/controller.pid"
MKDIR_ERROR_EXISTS=0 
create_folder "${JS7_CONTROLLER_DATA}"
create_folder "${JS7_CONTROLLER_LOGS}"
if [ "${JS7_CONTROLLER_LOGS}" != "${JS7_CONTROLLER_PID_FILE_DIR}" ]
then 
  create_folder "${JS7_CONTROLLER_PID_FILE_DIR}"
fi
create_folder "${JS7_CONTROLLER_DATA}/state"
create_folder "${JS7_CONTROLLER_CONFIG_DIR}/private"
copy_file "${JS7_CONTROLLER_CONFIG_DIR}/controller.conf-example" "${JS7_CONTROLLER_CONFIG_DIR}/controller.conf"
copy_file "${JS7_CONTROLLER_CONFIG_DIR}/private/private.conf-example" "${JS7_CONTROLLER_CONFIG_DIR}/private/private.conf"
if [ "${MKDIR_ERROR_EXISTS}" = 1 ]
then
  exit 4
fi


##########################
# set java options
echo "${JAVA_OPTIONS}" | grep "\-Xmx" >/dev/null 2>/dev/null
test $? -eq 0 || JAVA_OPTIONS="${JAVA_OPTIONS} -Xmx500m"
echo "${JAVA_OPTIONS}" | grep "\-Dfile.encoding" >/dev/null 2>/dev/null
test $? -eq 0 || JAVA_OPTIONS="${JAVA_OPTIONS} -Dfile.encoding=UTF-8"
LMAX_JAVA_OPTIONS="-Dlog4j2.contextSelector=org.apache.logging.log4j.core.async.AsyncLoggerContextSelector -Dlog4j2.asyncLoggerWaitStrategy=Block"
JAVA_OPTIONS="${JAVA_OPTIONS} ${LMAX_JAVA_OPTIONS}"


##########################
# set and check java
JAVABIN=`which java 2>/dev/null`
test ! -z "$JAVA_HOME" && test -x "$JAVA_HOME/bin/java" && JAVABIN="$JAVA_HOME/bin/java"
if [ -z "$JAVABIN" ]
then
  echo "...Java environment could not determined"
  echo "...please set JAVA_HOME properly"
  more_info 
  exit 5
fi


##########################
# ... and action
case "$1" in
    start)
        if [ "${CURRENT_USER}" = "${JS7_CONTROLLER_USER}" ]
        then
          start_controller
        else
          su - ${JS7_CONTROLLER_USER} -c "\"${JS7_CONTROLLER_STARTSCRIPT}\" $*"
        fi
        ;;
    start-docker|start_docker)
        start_controller_in_docker
        ;;
    stop)
        stop_controller
        ;;
    restart)
        if [ "${CURRENT_USER}" = "${JS7_CONTROLLER_USER}" ]
        then
          stop_controller
          start_controller
        else
          su - ${JS7_CONTROLLER_USER} -c "\"${JS7_CONTROLLER_STARTSCRIPT}\" $*"
        fi
        ;;
    abort)
        abort_controller
        ;;
    kill)
        kill_controller
        ;;
    status)
        shift
        status_controller "$@"
        ;;
    setup)
        echo "Directory for logs and pid files exists!"
        echo "Java version ok!"
        ;;
    cert)
        shift
        call_cert "@*"
        ;;
    *)
        usage
        ;;
esac

test -z "${JS7_CONTROLLER_EXIT}" && JS7_CONTROLLER_EXIT=0
exit $JS7_CONTROLLER_EXIT